/* eslint-disable default-case */
import { Member, Message } from 'eris';
import { Client, Command, RichEmbed } from '../class';

export default class Score extends Command {
  constructor(client: Client) {
    super(client);
    this.name = 'score';
    this.description = 'Pulls a hard score report for a member.';
    this.usage = `${this.client.config.prefix}score <member> <type: 'hard' | 'soft'> <reporting department: ex. Library of Code sp-us | Cloud Account Services>:<reason>\n${this.client.config.prefix}score notify <on | off>`;
    this.permissions = 0;
    this.guildOnly = true;
    this.enabled = true;
  }

  public async run(message: Message, args: string[]) {
    try {
      let member: Member;
      if (args[0] === 'notify') {
        member = message.member;
        if (!member) return this.error(message.channel, 'Member not found.');
        const score = await this.client.db.Score.findOne({ userID: message.author.id });
        if (!score) return this.error(message.channel, 'Score not calculated yet.');
        if (!score.notify) await this.client.db.Score.updateOne({ userID: message.author.id }, { $set: { notify: false } });
        switch (args[1]) {
          case 'on':
            await this.client.db.Score.updateOne({ userID: message.author.id }, { $set: { notify: true } });
            return this.success(message.channel, 'You will now be sent notifications whenever your score is hard-pulled.');
          case 'off':
            await this.client.db.Score.updateOne({ userID: message.author.id }, { $set: { notify: false } });
            return this.success(message.channel, 'You will no longer be sent notifications when your score is hard-pulled.');
          default:
            return this.error(message.channel, 'Invalid option. Valid options are `yes` and `no`.');
        }
      }
      if (!args[0] || !this.checkCustomPermissions(message.member, 2)) {
        member = message.member;
        if (!member) return this.error(message.channel, 'Member not found.');
      } else {
        member = this.client.util.resolveMember(args[0], this.mainGuild);
        if (!member) return this.error(message.channel, 'Member not found.');
        if (args[1] === 'hard') {
          if (args.length < 3) return this.client.commands.get('help').run(message, [this.name]);
          const name = args.slice(2).join(' ').split(':')[0];
          const reason = args.slice(2).join(' ').split(':')[1];
          const score = await this.client.db.Score.findOne({ userID: member.user.id });
          if (!score) return this.error(message.channel, 'Score not calculated yet.');
          await this.client.db.Score.updateOne({ userID: member.user.id }, { $addToSet: { inquiries: { name, reason, date: new Date() } } });
          if (score.notify === true) {
            await this.client.getDMChannel(member.user.id).then((chan) => {
              chan.createMessage(`__**Community Score - Hard Pull Notification**__\n*You have signed up to be notified whenever your hard score has been pulled. See \`?score\` for more information.*\n\n**Department/Service:** ${name.toUpperCase()}`);
            }).catch(() => {});
          }
        }
      }
      if (!member) return this.error(message.channel, 'Member not found.');
      const score = await this.client.db.Score.findOne({ userID: member.user.id });
      let totalScore = '0';
      let activityScore = '0';
      let moderationScore = '0';
      let roleScore = '0';
      let cloudServicesScore = '0';
      let otherScore = '0';
      let miscScore = '0';

      if (score) {
        if (score.total < 200) totalScore = '---';
        else if (score.total > 800) totalScore = '800';
        else totalScore = `${score.total}`;

        if (score.activity < 10) activityScore = '---';
        else if (score.activity > Math.floor((Math.log1p(5000 + 300 + 200 + 100) * 12))) activityScore = String(Math.floor((Math.log1p(5000 + 300 + 200 + 100) * 12)));
        else activityScore = `${score.activity}`;

        if (score.roles <= 0) roleScore = '---';
        else if (score.roles > 54) roleScore = '54';
        else roleScore = `${score.roles}`;

        moderationScore = `${score.moderation}`;

        if (score.other === 0) otherScore = '---';
        else otherScore = `${score.other}`;

        if (score.staff <= 0) miscScore = '---';
        else miscScore = `${score.staff}`;

        if (score.cloudServices === 0) cloudServicesScore = '---';
        else if (score.cloudServices > 10) cloudServicesScore = '10';
        else cloudServicesScore = `${score.cloudServices}`;
      } // else return this.error(message.channel, 'Community Score has not been calculated yet.');

      const embed = new RichEmbed();
      embed.setTitle('Community Score');
      embed.setAuthor(member.user.username, member.user.avatarURL);
      embed.setThumbnail(member.user.avatarURL);
      /* for (const role of member.roles.map((r) => this.mainGuild.roles.get(r)).sort((a, b) => b.position - a.position)) {
        if (role?.color !== 0) {
          embed.setColor(role.color);
          break;
        }
      } */
      if (score.inquiries?.length > 0) {
        let desc = '__**Hard Inquiries**__\n*These inquiries will fall off your report within 3 months, try to keep your hard inquiries to a minimum. If you want to file a dispute, please DM Ramirez.*\n\n';
        score.inquiries.forEach((inq) => {
          const testDate = (new Date(new Date(inq.date).setHours(2190)));
          if (testDate > new Date()) desc += `**Department/Service:** ${inq.name}\n**Reason:** ${inq.reason}\n**Date:** ${inq.date.toLocaleString('en-us')} ET\n\n`;
        });
        embed.setDescription(desc);
      }
      let color = '🔴';
      embed.setColor('FF0000');
      if (score.total >= 300) { color = '🟠'; embed.setColor('FFA500'); }
      if (score.total >= 500) { color = '🟡'; embed.setColor('FFFF00'); }
      if (score.total >= 700) { color = '🟢'; embed.setColor('66FF66'); }
      embed.addField('Total | 200 to 800', `${color} ${totalScore}`, true);
      embed.addField(`Activity | 10 to ${Math.floor(Math.log1p(5000 + 300 + 200 + 100) * 12)}`, activityScore || 'N/C', true);
      embed.addField('Roles | 1 to N/A', roleScore || 'N/C', true);
      embed.addField('Moderation | N/A to 2' || 'N/C', moderationScore, true);
      embed.addField('Cloud Services | N/A to 10+', cloudServicesScore || 'N/C', true);
      embed.addField('Other', otherScore || 'N/C', true);
      embed.addField('Misc', miscScore || 'N/C', true);
      if (score.lastUpdate) {
        embed.setFooter('Report last updated', this.client.user.avatarURL);
        embed.setTimestamp(score.lastUpdate);
      } else {
        embed.setFooter(this.client.user.username, this.client.user.avatarURL);
      }
      return message.channel.createMessage({ embed });
    } catch (err) {
      return this.client.util.handleError(err, message, this);
    }
  }
}
